@@ -127,6 +127,9 @@ def enumeration(*values, **kwargs):
 
     return type("Enumeration", (Enumeration,), attrs)()
 
+#: Specify whether events should be combined or collected as-is when a Document hold is in effect
+HoldPolicy = enumeration( "combine", "collect")
+
 #: Specify whether a dimension or coordinate is latitude or longitude
 LatLon = enumeration("lat", "lon")
 
@@ -142,6 +145,9 @@ LineCap = enumeration("butt", "round", "square")
 #: Specify the font style for rendering text
 FontStyle = enumeration("normal", "italic", "bold")
 
+#: Specify the vertical alignment for rendering text
+VerticalAlign = enumeration("top", "middle", "bottom")
+
 #: Specify the horizontal alignment for rendering text
 TextAlign = enumeration("left", "right", "center")
 
@@ -185,6 +191,10 @@ VerticalLocation = enumeration("above", "below")
 #: Specify a horizontal location in plot layouts
 HorizontalLocation = enumeration("left", "right")
 
+#: Specify an attachment for tooltips
+TooltipAttachment = enumeration("horizontal", "vertical",
+                                "left", "right", "above", "below")
+
 #: Specify a named dashing patter for stroking lines
 DashPattern = enumeration("solid", "dashed", "dotted", "dotdash", "dashdot")
 
@@ -192,7 +202,7 @@ DashPattern = enumeration("solid", "dashed", "dotted", "dotdash", "dashdot")
 ButtonType = enumeration("default", "primary", "success", "warning", "danger", "link")
 
 #: Specify one of the 137 named CSS colors
-NamedColor = enumeration(*colors.__colors__, case_sensitive=False)
+NamedColor = enumeration(*colors.named.__all__, case_sensitive=False)
 
 #: Specify the name of a palette from :ref:`bokeh.palettes`
 Palette = enumeration(*palettes.__palettes__)
@@ -222,11 +232,6 @@ RenderLevel = enumeration("image", "underlay", "glyph", "annotation", "overlay")
 #: Specify a render mode for renderers that support both Canvas or CSS rendering
 RenderMode = enumeration("canvas", "css")
 
-# TODO (bev) Aggregation can probably be deprecated and removed
-
-#: Specify an aggregation type
-Aggregation = enumeration("sum", "mean", "count", "nunique", "median", "min", "max")
-
 #: Specify a start/end value
 StartEnd = enumeration("start", "end")
 
