@@ -38,6 +38,12 @@ class Transform(Model):
 class CustomJSTransform(Transform):
     ''' Apply a custom defined transform to data.
 
+    .. warning::
+        The explicit purpose of this Bokeh Model is to embed *raw JavaScript
+        code* for a browser to execute. If any part of the code is derived
+        from untrusted user inputs, then you must take appropriate care to
+        sanitize the user input prior to passing to Bokeh.
+
     '''
 
     @classmethod
